from .api.check_for_imports import check
from .api.cli_utils import print_header, command
from .api.janet_record import JanetRecord
from .api.autocompleter import SimpleCompleter

from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.events import EVENT_JOB_ERROR, EVENT_JOB_EXECUTED
import atexit

from colorama import Fore
import argparse
import os
import time
import logging
import signal
import readline


logging.getLogger('apscheduler.scheduler').setLevel('ERROR')
logging.getLogger('apscheduler.executors.default').propagate = False

parser = argparse.ArgumentParser(description='Start a janet project manager')
parser.add_argument('-p', '--project_path',
					nargs='?',
					type=str,
					help='path to your project',
					default=".",
					const=".")


parser.add_argument('-s', '--script',
					nargs='?',
					type=str,
					help='specify the entry point for your project',
					default="",
					const="")

args = parser.parse_args()


project_path = os.path.abspath(args.project_path)
entry_point = args.script

janetrecord = JanetRecord(project_path)


scheduler = BackgroundScheduler()
scheduler.add_job(func=check , args=[project_path, janetrecord], trigger="interval", seconds=2)
scheduler.add_job(func=janetrecord.save_janetrecord, trigger="interval", seconds=60)
## kill the scheduled process when janet shuts down
atexit.register(lambda: scheduler.shutdown())


# Register our completer function
readline.set_completer(SimpleCompleter(['exit',
										'install',
										'run',
										'change-entrypoint',
										'kill',
										'menu']).complete)
# Use the tab key for completion
readline.parse_and_bind('tab: complete')


if __name__ == "__main__":
	process = None
	print_header()
	scheduler.start()

	while True:
		cmd = input(Fore.WHITE+'>>> ')
		process, entry_point = command(cmd, process, entry_point, project_path)

	


import os
import importlib
import subprocess
from subprocess import Popen, PIPE
import sys
import time


def list_files(startpath):
	file_list = []
	env_name = sys.executable.split('/')[-3]
	for root, dirs, files in os.walk(startpath):
		if "/" in root:
			dir_name = root.split("/")[1]
			if dir_name.startswith("."):
				continue
			elif env_name in root:
				continue
  			#REMOVE LATER
			if "api" in root:
				continue
		for f in files:
			if f.endswith(".py") :
				file_list.append(f"{root}/{f}")

	return file_list

def update_requirements(janetrecord):
	spit_packages = [sys.executable, "-m", "pip", "freeze"]
	pip_freeze = subprocess.run(spit_packages, universal_newlines=True, stdout=subprocess.PIPE)

	frozen_packages = pip_freeze.stdout
	with open("_temp_janet_file.txt", "w") as temp_file:
		temp_file.write(frozen_packages)

	with open("requirements.txt", "w") as requirements_file:
		for line in open("_temp_janet_file.txt"):
			line_splitted = line.split("==")[0].lower()
			if line_splitted not in janetrecord.janet_requirements:
				requirements_file.writelines(line)

	os.remove("_temp_janet_file.txt")



def install(modules):
	cmd = [sys.executable, "-m", "pip", "install"]
	cmd.extend(modules)
	
	pip_install = subprocess.run(cmd, universal_newlines=True, stdout=subprocess.PIPE)
	


def check_module_in_pip(module):
	cmd = [sys.executable, '-m', 'pip', 'search', module]
	result = subprocess.run(cmd, capture_output=True)
	return len(result.stdout) > 0


def resolve_imports(lines):
	modules_to_resolve = []

	for line in lines:
		if( ("import" in line) and
			 (not line.startswith('#')) and 
			 	(not line.endswith("import")) ):
			line_formated = line.replace("import","")
			line_split= line_formated.split(",")

			for imported_module in line_split:
				as_keyword_index = imported_module.find(" as ")
				if as_keyword_index != -1:
					imported_module = imported_module[0:as_keyword_index]
					
				imported_module = imported_module.replace(" ","")
				imported_module = imported_module.split(".")[0]
				imported_module = imported_module.rstrip("\n")
				modules_to_resolve.append(imported_module)

	return modules_to_resolve



def check(project_dir, janetrecord):
	files = list_files(project_dir)
	modules = []

	for file in files:
		exists = janetrecord.it_exists(file)

		if not exists:
			janetrecord.store_last_modified(file)
		else:
			last_modified = janetrecord.get_last_modified_time(file)

			if last_modified > janetrecord.get_last_modified_from_records(file):
				janetrecord.store_last_modified(file)
				code = open(file, "r")
				lines = code.readlines()
				
				modules_to_resolve = resolve_imports(lines)

				if len(modules_to_resolve):
					modules.extend(modules_to_resolve)


	uninstalled_modules = []

	for module in modules:
		
		if (module == " ") or (module == ""):
			continue
		elif module in os.listdir(project_dir):
				continue


		found = importlib.find_loader(module)
		if not found:
			is_in_pip = check_module_in_pip(module)
			if not is_in_pip:
				print(f"{module} is neither a local import nor a pip package, skipping ...")
				continue
			else:
				uninstalled_modules.append(module)
			

	

	if len(uninstalled_modules):
		print('You have the following uninstalled module(s)')
		for idx, mod in enumerate(uninstalled_modules):
			print(f"	{idx}: {mod}")
		print("I'll take care of it!")
		print()
 
		install(uninstalled_modules)	
		print()
		print("Installation completed")
		print("press enter to show cli..")

	update_requirements(janetrecord)






from .check_for_imports import list_files
import subprocess
import psutil
import sys
import os


def call_run_subprocess(file, args=None):
	cmd = [sys.executable, file]
	if args is not None:
		cmd.extend(args)
	process = subprocess.Popen(cmd)

	return process



def run_code(project_dir, entry_point, debug=False):
	files = list_files(project_dir)
	entry_point_file = ''
	
	entry_point_splitted = entry_point.split(".")

	if len(entry_point_splitted) > 1:
		if entry_point_splitted[1] == "py":
			entry_point = f"{entry_point}"
		else:
			entry_point = f"{entry_point_splitted[0]}.py"
	else:
		entry_point = f"{entry_point_splitted[0]}.py"


	for file in files:
		if entry_point in file:
			entry_point_file += file
			break
	if len(entry_point_file) == 0:
		print("Invalid entry point")
		return None

	process = call_run_subprocess(entry_point_file)

	return process





from .run import run_code
from .check_for_imports import install

from colorama import Fore
import argparse
import os
import time
import logging
import signal

from pyfiglet import Figlet



def print_header():
	f = Figlet(font='slant')
	print(f.renderText('JANET'))
	print_cmd()

def print_cmd():
    print(Fore.WHITE + 'Cool stuff you could do:')
    print(Fore.GREEN+ '	1. run (to run the selected entry point, running again will kill the current python process)')
    print(Fore.GREEN+ '	2. install <PACKAGE_1> <PACKAGE_2> ... (install packages manually)')
    print(Fore.GREEN+ '	3. change-entrypoint (change the script that janet will run)')
    print(Fore.GREEN+ '	4. kill (to manually end a run)')
    print(Fore.GREEN+ '	5. exit (to stop janet)')
    print(Fore.GREEN+ '	6. menu')
    print(Fore.GREEN+ '	\nUse TAB to autocomplete commands')
    print(Fore.WHITE)




def command(command, process, entry_point, project_path):
	command_no_spaces = command.replace(" ", "")
	command_splited = command.split(" ")

	if command_no_spaces.lower() == 'exit':
		if process is not None:
			process.send_signal(signal.SIGINT)
		exit()

	elif command_splited[0].lower() == "install":
		modules = []
		for arg in command_splited[1:]:
			if (arg == " ") or (arg == ""):
				continue
			else:
				modules.append(arg)
		install(modules)
		return process, entry_point


	elif command_no_spaces.lower() == 'run':
		if process is not None:
			print("you have to kill the current process before starting a new one")
			process.send_signal(signal.SIGINT)

		process = run_code(project_path, entry_point, debug=False)
		return process, entry_point

	elif command_no_spaces.lower() == 'change-entrypoint':
		new_entry_point = ''
		while len(new_entry_point) == 0:
			new_entry_point = input(Fore.WHITE+'New entry point ? ')
			new_entry_point = new_entry_point.replace(" ","")
		entry_point = new_entry_point
		return process, entry_point


	elif command_no_spaces.lower() == 'kill':
		if process is not None:
			process.send_signal(signal.SIGINT)
		else:
			print("Nothing to kill")

		return None, entry_point

	elif command_no_spaces.lower() == 'menu':
		print_cmd()
		return process, entry_point

	else:
		return process, entry_point




		